<?php

class schedulerPDF {

	public $offsetTop = 15;
	public $offsetBottom = 10;
	public $offsetLeft = 10;
	public $offsetRight = 10;

	// header height of day container in month mode
	public $monthDayHeaderHeight = 6;
	// header height in month mode
	public $monthHeaderHeight = 8;
	// height of month name container in year mode
	public $yearMonthHeaderHeight = 8;
	// height of row in agenda mode
	public $agendaRowHeight = 6;
	// height of header in day and week mode
	public $dayTopHeight = 6;
	// width of left scale in day and week mode
	public $dayLeftWidth = 16;
	
	public $strip_tags = false;

	/* header/footer printing */
	private $header = false;
	private $footer = false;
	public $headerImgHeight = 40;
	public $footerImgHeight = 40;
	public $headerImg = './header.png';
	public $footerImg = './footer.png';

	// font size settings
	public $monthHeaderFontSize = 9;
	public $monthDayHeaderFontSize = 8;
	public $monthEventFontSize = 7;
	public $yearHeaderFontSize = 8;
	public $yearFontSize = 8;
	public $agendaFontSize = 8;
	public $dayHeaderFontSize = 7;
	public $dayScaleFontSize = 8;
	public $dayEventHeaderFontSize = 7;
	public $dayEventBodyFontSize = 7;
	public $todayFontSize = 11;

	private $profile = 'color';
	private $orientation = 'P';
	private $mode;
	private $today;

	public $lineColor;
	public $bgColor;
	public $dayHeaderColor;
	public $dayBodyColor;
	public $dayHeaderColorInactive;
	public $dayBodyColorInactive;
	public $headerTextColor;
	public $textColor;
	public $eventTextColor;
	public $eventBorderColor;
	public $eventColor;
	public $todayTextColor;
	public $scaleColorOne;
	public $scaleColorTwo;
	public $timelineCellBg;

	public function printScheduler($xml) {
		$this->renderData($xml);
		$this->renderScales($xml);
		$this->renderEvents($xml);

		if ($this->header !== false) {
			$this->offsetTop += $this->headerImgHeight;
		}
		if ($this->footer !== false) {
			$this->offsetBottom += $this->footerImgHeight;
		}
		$this->wrapper = new pdfWrapper($this->offsetTop, $this->offsetRight, $this->offsetBottom, $this->offsetLeft, $this->orientation);
		switch ($this->mode) {
			case 'month':
				$this->orientation = 'L';
				$this->printMonth();
				break;
			case 'year':
				$this->orientation = 'L';
				$this->printYear();
				break;
			case 'agenda':
				$this->orientation = 'P';
				$this->printAgenda();
				break;
			case 'timeline':
				$this->orientation = 'L';
				$this->printTimeline();
				break;
			case 'day':
			case 'unit':
			case 'week':
			default:
				$this->orientation = 'P';
				$this->printDayWeek();
				break;
		}

		$this->wrapper->pdfOut();
	}

	// parses options profile, orientation, header and footer from xml 
	private function renderData($data) {
		if (isset($data->attributes()->profile)) {
			$this->profile = (string) $data->attributes()->profile;
		}
		if (isset($data->attributes()->orientation)) {
			$this->orientation = (string) $data->attributes()->orientation;
		}
		if (isset($data->attributes()->header)) {
			$this->header = (string) $data->attributes()->header;
		}
		if (!file_exists($this->headerImg)) {
			$this->header = false;
		}
		if (isset($data->attributes()->footer)) {
			$this->footer = (string) $data->attributes()->footer;
		}
		if (!file_exists($this->footerImg)) {
			$this->footer = false;
		}
		$this->setProfile();
	}


	// sets color profile
	private function setProfile() {
		switch ($this->profile) {
			case 'color':
				$this->lineColor = '586A7E';
				$this->bgColor = 'C2D5FC';
				$this->dayHeaderColor = 'EBEFF4';
				$this->dayBodyColor = 'FFFFFF';
				$this->dayHeaderColorInactive = 'E2E3E6';
				$this->dayBodyColorInactive = 'ECECEC';
				$this->headerTextColor = '2F3A48';
				$this->textColor = '2F3A48';
				$this->eventTextColor = '887A2E';
				$this->eventTextColor = '000000';
				$this->eventBorderColor = 'B7A543';
				$this->eventColor = 'FFE763';
				$this->todayTextColor = '000000';
				$this->scaleColorOne = 'FCFEFC';
				$this->scaleColorTwo = 'DCE6F4';
				$this->yearDayColor = 'EBEFF4';
				$this->yearDayColorInactive = 'd6d6d6';
				$this->timelineCellBg = 'FFFFFF';
				break;
			case 'gray':
				$this->lineColor = '666666';
				$this->bgColor = 'D3D3D3';
				$this->dayHeaderColor = 'EEEEEE';
				$this->dayBodyColor = 'FFFFFF';
				$this->dayHeaderColorInactive = 'E3E3E3';
				$this->dayBodyColorInactive = 'ECECEC';
				$this->headerTextColor = '383838';
				$this->textColor = '000000';
				$this->eventTextColor = '000000';
				$this->eventBorderColor = '9F9F9F';
				$this->eventColor = 'DFDFDF';
				$this->todayTextColor = '000000';
				$this->scaleColorOne = 'E4E4E4';
				$this->scaleColorTwo = 'FDFDFD';
				$this->yearDayColor = 'EBEFF4';
				$this->yearDayColorInactive = 'E2E3E6';
				$this->timelineCellBg = 'FFFFFF';
				break;
			case 'bw':
				$this->lineColor = '000000';
				$this->bgColor = 'FFFFFF';
				$this->dayHeaderColor = 'FFFFFF';
				$this->dayBodyColor = 'FFFFFF';
				$this->dayHeaderColorInactive = 'FFFFFF';
				$this->dayBodyColorInactive = 'FFFFFF';
				$this->headerTextColor = '000000';
				$this->textColor = '000000';
				$this->eventTextColor = '000000';
				$this->eventBorderColor = '000000';
				$this->eventColor = 'FFFFFF';
				$this->todayTextColor = '000000';
				$this->scaleColorOne = 'FFFFFF';
				$this->scaleColorTwo = 'FFFFFF';	
				$this->yearDayColor = 'FFFFFF';
				$this->yearDayColorInactive = 'FFFFFF';
				$this->timelineCellBg = 'FFFFFF';
				break;
		}
	}


	// render scales
	private function renderScales($xml) {
		$scales = $xml->scale;
		$this->mode = (string) $scales->attributes()->mode;
		$this->today = $this->strip((string) $scales->attributes()->today);

		switch ($this->mode) {
			case 'month':
				foreach ($scales->x->column as $text) {
					$this->topScales[] = $this->strip((string) $text);
				}
				foreach ($scales->row as $text) {
					$week = explode("|", $text);
					for ($i = 0; $i < count($week); $i++) {
						$week[$i] = $this->strip($week[$i]);
					}
					$this->dayHeader[] = $week;
				}
				break;

			case 'year':
				foreach ($scales->month as $month) {
					$monthArr = Array();
					$monthArr['label'] = $this->strip((string) $month->attributes()->label);
					foreach ($month->column as $col) {
						$monthArr['columns'][] = $this->strip((string) $col);
					}

					foreach ($month->row as $row) {
						$row = $this->strip($row);
						$row = explode("|", $row);
						$monthArr['rows'][] = $row;
					}
					$this->yearValues[] = $monthArr;
				}
				break;
			case 'agenda':
				$this->agendaHeader[0] = $this->strip((string) $scales->column[0]);
				$this->agendaHeader[1] = $this->strip((string) $scales->column[1]);
				break;

			case 'timeline':
				foreach ($scales->x->column as $col) {
					$this->columnHeader[] = $this->strip((string) $col);
				}
				foreach ($scales->y->row as $row) {
					$this->rowHeader[] = $this->strip((string) $row);
					if (isset($row->attributes()->bg)) {
						$bgs = $this->strip((string) $row->attributes()->bg);
						$bgs = explode("|", $bgs);
					} else {
						$bgs = false;
					}
					$this->cellColors[] = $bgs;
				}
				break;

			case 'day':
			case 'unit':
			case 'week':
			default:
				foreach ($scales->x->column as $col) {
					$this->columnHeader[] = $this->strip((string) $col);
				}
				foreach ($scales->y->row as $row) {
					$this->rowHeader[] = $this->strip((string) $row);
				}
				break;
		}
	}


	// parses events from xml
	private function renderEvents($xml) {
		$this->events = Array();
		switch ($this->mode) {
			case 'month':
				foreach ($xml->event as $ev) {
					$event['week'] = (int) $ev->attributes()->week - 1;
					$event['day'] = (int) $ev->attributes()->day;
					$event['x'] = (float) $ev->attributes()->x;
					$event['y'] = (float) $ev->attributes()->y;
					$event['width'] = (float) $ev->attributes()->width;
					$event['height'] = (float) $ev->attributes()->height;
					$event['type'] = (string) $ev->attributes()->type;
					$event['text'] = $this->strip((string) $ev->body);
					$event['text_color'] = $this->parseColor((string) $ev->body->attributes()->color);
					$event['color'] = $this->parseColor((string) $ev->body->attributes()->backgroundColor);
					$this->events[] = $event;
				}
				break;
			case 'year':
				foreach ($xml->event as $ev) {
					$week = (int) $ev->attributes()->week;
					$day = (int) $ev->attributes()->day;
					$month = (int) $ev->attributes()->month;
					$this->events[] = $month.'_'.$week.'_'.$day;
				}
				break;
			case 'agenda':
				foreach ($xml->event as $ev) {
					$head = $this->strip((string) $ev->head);
					$body = $this->strip((string) $ev->body);
					$this->events[] = Array('head' => $head, 'body' => $body);
				}
				break;
			case 'day':
			case 'week':
			case 'unit':
			default:
				foreach ($xml->event as $ev) {
					$event['head'] = $this->strip((string) $ev->header);
					$event['body'] = $this->strip((string) $ev->body);
					$event['x'] = (float) $ev->attributes()->x;
					$event['y'] = (float) $ev->attributes()->y;
					$event['width'] = (float) $ev->attributes()->width;
					$event['height'] = (float) $ev->attributes()->height;
					$event['text_color'] = $this->parseColor((string) $ev->body->attributes()->color);
					$event['color'] = $this->parseColor((string) $ev->body->attributes()->backgroundColor);
					$this->events[] = $event;
				}
		}
	}


	private function parseColor($colorStr) {
		$color_ind = array();
		preg_match("/rgb\(\s*(\d{1,3})\s*,\s*(\d{1,3})\s*,\s*(\d{1,3})\s*\)/", $colorStr, $color_ind);
		if (count($color_ind) > 0) {
			$r = (String) dechex($color_ind[1]);
			$r = (strlen($r) > 1) ? $r : '0'.$r;
			$g = (String) dechex($color_ind[2]);
			$g = (strlen($g) > 1) ? $g : '0'.$g;
			$b= (String) dechex($color_ind[3]);
			$b = (strlen($b) > 1) ? $b : '0'.$b;
			return $r.$g.$b;
		}
		preg_match("/#([0-9a-fA-F]{6})/", $colorStr, $color_ind);
		if (count($color_ind) == 2) {
			return $color_ind[1];
		}
		return false;
	}
	

	private function printMonth() {
		if ($this->header !== false) {
			$this->wrapper->setHeader($this->headerImg, $this->headerImgHeight);
		}
		if ($this->footer !== false) {
			$this->wrapper->setFooter($this->footerImg, $this->footerImgHeight);
		}
		$this->wrapper->drawMonthHeader($this->orientation, $this->topScales, $this->monthHeaderHeight, $this->bgColor, $this->lineColor, $this->monthHeaderFontSize);
		$this->wrapper->drawToday($this->today, $this->todayFontSize);
		$this->wrapper->drawMonthGrid($this->dayHeader, $this->monthDayHeaderHeight, $this->dayHeaderColor, $this->dayBodyColor, $this->dayHeaderColorInactive, $this->dayBodyColorInactive, $this->monthDayHeaderFontSize);
		$this->wrapper->drawMonthEvents($this->events, $this->eventColor, $this->eventBorderColor, $this->eventTextColor, $this->monthEventFontSize, $this->profile);
	}


	private function printYear() {
		if ($this->header !== false) {
			$this->wrapper->setHeader($this->headerImg, $this->headerImgHeight);
		}
		if ($this->footer !== false) {
			$this->wrapper->setFooter($this->footerImg, $this->footerImgHeight);
		}
		$this->wrapper->drawYear($this->orientation, $this->yearValues, $this->events, $this->yearMonthHeaderHeight, $this->bgColor, $this->lineColor, $this->yearDayColor, $this->yearDayColorInactive, $this->eventColor, $this->yearHeaderFontSize, $this->yearFontSize);
		$this->wrapper->drawToday($this->today, $this->todayFontSize);
	}


	private function printAgenda() {
		if ($this->header !== false) {
			$this->wrapper->setHeader($this->headerImg, $this->headerImgHeight);
		}
		if ($this->footer !== false) {
			$this->wrapper->setFooter($this->footerImg, $this->footerImgHeight);
		}
		$this->wrapper->drawAgenda($this->agendaHeader, $this->events, $this->agendaRowHeight, $this->bgColor, $this->lineColor, $this->scaleColorOne, $this->scaleColorTwo, $this->agendaFontSize, $this->today, $this->todayFontSize);
	}


	private function printDayWeek() {
		if ($this->header !== false) {
			$this->wrapper->setHeader($this->headerImg, $this->headerImgHeight);
		}
		if ($this->footer !== false) {
			$this->wrapper->setFooter($this->footerImg, $this->footerImgHeight);
		}
		$this->wrapper->drawDayWeekHeader($this->columnHeader, $this->rowHeader, $this->dayTopHeight, $this->dayLeftWidth, $this->bgColor, $this->lineColor, $this->scaleColorOne, $this->scaleColorTwo, $this->dayHeaderFontSize, $this->dayScaleFontSize, $this->profile);
		$this->wrapper->drawToday($this->today, $this->todayFontSize);
		$this->wrapper->drawDayWeekEvents($this->events, $this->eventColor, $this->eventBorderColor, $this->eventTextColor, $this->dayEventHeaderFontSize, $this->dayEventBodyFontSize, $this->profile);
	}

	private function printTimeline() {
		if ($this->header !== false) {
			$this->wrapper->setHeader($this->headerImg, $this->headerImgHeight);
		}
		if ($this->footer !== false) {
			$this->wrapper->setFooter($this->footerImg, $this->footerImgHeight);
		}
		$this->wrapper->drawTimelineHeader($this->columnHeader, $this->rowHeader, $this->cellColors, $this->dayTopHeight, $this->dayLeftWidth, $this->bgColor, $this->lineColor, $this->timelineCellBg, $this->dayHeaderFontSize, $this->dayScaleFontSize, $this->profile, $this->orientation);
		$this->wrapper->drawToday($this->today, $this->todayFontSize);
		$this->wrapper->drawTimelineEvents($this->events, $this->eventColor, $this->eventBorderColor, $this->eventTextColor, $this->monthEventFontSize, $this->profile);
	}

	private function strip($param) {
		if ($this->strip_tags == true) {
			$param = strip_tags($param);
		}
		return $param;
	}

}


?>